#! /usr/bin/env python

import getopt, jabber, socket, string, sys, time, timeoutsocket, urllib2

class JabberClient:

    def __init__( self, hostname, username, password ):
        self.client = jabber.Client( hostname )

        self.client.connect()
        
        self.client.setPresenceHandler( self.handlePresence )
        
        self.client.auth( username, password, 'surveillance' )
        self.client.requestRoster()
        self.client.sendInitPresence()
        
    def __del__( self ):
        self.client.disconnect()

    def setAvailable( self ):
        self.client.sendInitPresence()
        
    def setUnavailable( self ):
        presence = jabber.Presence()
        presence.setShow( 'xa' )
        presence.setStatus( 'Service unavailable' )
        self.client.send( presence )
        
    def handlePresence( self, client, presence ):
        type = presence.getType()
        who = presence.getFrom().getStripped()

        if ( type == 'subscribe' ):
            self.client.send( jabber.Presence( to = who, type = 'subscribed' ) )
            self.client.send( jabber.Presence( to = who, type = 'subscribe' ) )

        elif ( type == 'unsubscribe' ):
            self.client.send( jabber.Presence( to = who, type = 'unsubscribed' ) )
            self.client.send( jabber.Presence( to = who, type = 'unsubscribe' ) )

    def processEvents( self ):
        self.client.process( 1 )

    def sendMessageToAll( self, text ):
        roster = self.client.requestRoster()
        users = roster.getSummary()
        for user in users.keys():
            message = jabber.Message( user, text )
            self.client.send( message )
            
class Checker:

    def __init__( self, url ):
        self.url = url
        
        timeoutsocket.setDefaultSocketTimeout( 30 )
        
        passwordMgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        passwordMgr.add_password( None, 'www.somewhere.com', 'user', 'password' )

        authHandler = urllib2.HTTPBasicAuthHandler( passwordMgr )
        opener = urllib2.build_opener( authHandler )
        urllib2.install_opener( opener )

    def performCheck( self ):    
        try:
            result = urllib2.urlopen( self.url )
            result.close()
            return True 
        except:
            return False

def usage():
    print "Usage: jabberSurveillance.py"
    print " Options:"
    print "  -c, --check=URL to check"
    print "      --help"
    print "  -h, --host=HOSTNAME (default: localhost)"
    print "  -i, --interval=time between checks (default 60s)"
    print "  -u, --user=USERNAME"
    print "  -p, --password=PASSWORD"

def main():
    options = "c:h:i:u:p:"
    long_options = [ "help", "check=", "host=", "interval=", "user=", "password=" ]

    try:
        opts, args = getopt.getopt( sys.argv[1:], options, long_options )
    except getopt.GetoptError:
        usage()
        sys.exit( 2 )

    check = ""
    host = "localhost"
    interval = 60
    user = ""
    password = ""

    for option, arg in opts:

        if ( option == "--help" ):
            usage()
            sys.exit()

        if ( option in ( "-c", "--check" ) ):
            check = arg

        if ( option in ( "-h", "--host" ) ):
            host = arg

        if ( option in ( "-i", "--interval" ) ):
            interval = int( arg )

        if ( option in ( "-u", "--user" ) ):
            user = arg

        if ( option in ( "-p", "--password" ) ):
            password = arg

    if ( ( check == "" ) or ( user == "" ) or ( password == "" ) ):
        usage()
        sys.exit( 2 )
    else:
        available = True
        client = JabberClient( host, user, password )
        checker = Checker( check )
        while ( True ):
            client.processEvents()
            if checker.performCheck():
                if ( not available ):
                    available = True
                    client.sendMessageToAll( 'The page with the URL ' + check + ' is available again since ' + time.ctime() + '.' )
                    client.setAvailable()
            else:
                if ( available ):
                    available = False
                    client.sendMessageToAll( 'The page with the URL ' + check + ' was unavailable at ' + time.ctime() + '.' )
                    client.setUnavailable()

            time.sleep( interval )
                
try:
    if __name__ == "__main__":
        main()
except KeyboardInterrupt:
    pass
